@@ -775,7 +775,10 @@ def make_global_scope_command(cmd_name):
     """
     def wrapper(path, *args, **kwargs):
         implementation = kwargs.pop('implementation', None)
-        if implementation is None: implementation = best_implementation(cmd_name, path)
+        if implementation is None:
+            implementation = best_implementation(cmd_name, path)
+        elif is_string(implementation):
+            implementation = implementations[implementation]
         return getattr(implementation, cmd_name)(path, *args, **kwargs)
     return wrapper
         
@@ -1015,8 +1018,8 @@ def diff(left, right, stat=False, implementation=None):
         if not has_protocol(path):
             raise NotImplementedError('diff does not support local paths yet: {0}'.format(path))
 
-    contents_left = listdir_recursive(left)
-    contents_right = listdir_recursive(right)
+    contents_left = listdir_recursive(left, implementation=implementation)
+    contents_right = listdir_recursive(right, implementation=implementation)
 
     if stat:
         paths_left = [ n.path for n in contents_left ]
